@@ -16,7 +16,8 @@ has id => ( is => 'ro', isa => Str, required => 1 );
 has listen => ( is => 'ro', isa => HashRef );
 
 has _ws => ( is => 'ro', isa => InstanceOf ['Pcore::WebSocket::Protocol::raw'], init_arg => undef );
-has _cb => ( is => 'ro', isa => HashRef, init_arg => undef );
+has _cb      => ( is => 'ro', isa => HashRef,  init_arg => undef );
+has _conn_cb => ( is => 'ro', isa => ArrayRef, init_arg => undef );
 
 our $_MSG_ID = 0;
 
@@ -90,6 +91,10 @@ sub _cmd ( $self, $cmd, @args ) {
 }
 
 sub _connect ( $self, $cb ) {
+    push $self->{_conn_cb}->@*, $cb;
+
+    return if $self->{_conn_cb}->@* > 1;
+
     weaken $self;
 
     Pcore::WebSocket->connect_ws(
@@ -99,6 +104,14 @@ sub _connect ( $self, $cb ) {
         connect_timeout  => 1000,
         tls_ctx          => undef,
         on_connect_error => sub ($res) {
+
+            # call callbacks
+            if ( my $callbacks = delete $self->{_conn_cb} ) {
+                for my $cb ( $callbacks->@* ) {
+                    AE::postpone { $cb->() };
+                }
+            }
+
             return;
         },
         on_connect => sub ( $ws, $headers ) {
@@ -124,13 +137,25 @@ sub _connect ( $self, $cb ) {
                 return;
             };
 
-            $cb->();
+            # call callbacks
+            if ( my $callbacks = delete $self->{_conn_cb} ) {
+                for my $cb ( $callbacks->@* ) {
+                    AE::postpone { $cb->() };
+                }
+            }
 
             return;
         },
         on_disconnect => sub ( $ws, $status ) {
             undef $self->{_ws};
 
+            # call pending callbacks
+            if ( my $callbacks = delete $self->{_cb} ) {
+                for my $cb ( values $callbacks->%* ) {
+                    $cb->[1]->( $cb->[0], undef );
+                }
+            }
+
             return;
         },
     );
